@@ -4,13 +4,19 @@ import os
 import re
 import socket
 import sys
+import traceback
+import webbrowser
+import functools
+from typing import Set
 
 import django
 from django.conf import settings
 from django.core.management.base import BaseCommand, CommandError, SystemCheckError
+from django.core.management.color import color_style
 from django.core.servers.basehttp import get_internal_wsgi_application
 from django.dispatch import Signal
 from django.utils.autoreload import get_reloader
+from django.views import debug as django_views_debug
 
 try:
     if 'whitenoise.runserver_nostatic' in settings.INSTALLED_APPS:
@@ -21,6 +27,23 @@ try:
 except ImportError:
     USE_STATICFILES = False
 
+try:
+    from werkzeug import run_simple
+    from werkzeug.debug import DebuggedApplication
+    from werkzeug.serving import WSGIRequestHandler as _WSGIRequestHandler
+    from werkzeug.serving import make_ssl_devcert
+    from werkzeug._internal import _log  # type: ignore
+    from werkzeug import _reloader
+    HAS_WERKZEUG = True
+except ImportError:
+    HAS_WERKZEUG = False
+
+try:
+    import OpenSSL  # NOQA
+    HAS_OPENSSL = True
+except ImportError:
+    HAS_OPENSSL = False
+
 from django_extensions.management.technical_response import null_technical_500_response
 from django_extensions.management.utils import RedirectHandler, has_ipdb, setup_logger, signalcommand
 from django_extensions.management.debug_cursor import monkey_patch_cursordebugwrapper
@@ -50,12 +73,57 @@ DEFAULT_POLLER_RELOADER_INTERVAL = getattr(settings, 'RUNSERVERPLUS_POLLER_RELOA
 DEFAULT_POLLER_RELOADER_TYPE = getattr(settings, 'RUNSERVERPLUS_POLLER_RELOADER_TYPE', 'auto')
 
 logger = logging.getLogger(__name__)
+_error_files = set()  # type: Set[str]
+
+
+if HAS_WERKZEUG:
+    # Monkey patch the reloader to support adding more files to extra_files
+    for name, reloader_loop_klass in _reloader.reloader_loops.items():
+        class WrappedReloaderLoop(reloader_loop_klass):  # type: ignore
+            def __init__(self, *args, **kwargs):
+                super().__init__(*args, **kwargs)
+                self._extra_files = self.extra_files
+
+            @property
+            def extra_files(self):
+                return self._extra_files.union(_error_files)
+
+            @extra_files.setter
+            def extra_files(self, extra_files):
+                self._extra_files = extra_files
+
+        _reloader.reloader_loops[name] = WrappedReloaderLoop
 
 
 def gen_filenames():
     return get_reloader().watched_files()
 
 
+def check_errors(fn):
+    # Inspired by https://github.com/django/django/blob/master/django/utils/autoreload.py
+    @functools.wraps(fn)
+    def wrapper(*args, **kwargs):
+        try:
+            return fn(*args, **kwargs)
+        except Exception:
+            _exception = sys.exc_info()
+
+            _, ev, tb = _exception
+
+            if getattr(ev, 'filename', None) is None:
+                # get the filename from the last item in the stack
+                filename = traceback.extract_tb(tb)[-1][0]
+            else:
+                filename = ev.filename
+
+            if filename not in _error_files:
+                _error_files.add(filename)
+
+            raise
+
+    return wrapper
+
+
 class Command(BaseCommand):
     help = "Starts a lightweight Web server for development."
 
@@ -112,7 +180,7 @@ class Command(BaseCommand):
                                  "because Django debug pages tries to call the function and unintentionally shuts down "
                                  "the Werkzeug server.")
         parser.add_argument("--nopin", dest="nopin", action="store_true", default=False,
-                            help="Disable the PIN in werkzeug. USE IT WISELY!"),
+                            help="Disable the PIN in werkzeug. USE IT WISELY!")
 
         if USE_STATICFILES:
             parser.add_argument('--nostatic', action="store_false", dest='use_static_handler', default=True,
@@ -135,10 +203,6 @@ class Command(BaseCommand):
 
         os.environ['RUNSERVER_PLUS_SHOW_MESSAGES'] = '1'
 
-        # Do not use default ending='\n', because StreamHandler() takes care of it
-        if hasattr(self.stderr, 'ending'):
-            self.stderr.ending = None
-
         setup_logger(logger, self.stderr, filename=options['output_file'])  # , fmt="[%(name)s] %(message)s")
         logredirect = RedirectHandler(__name__)
 
@@ -187,8 +251,7 @@ class Command(BaseCommand):
                 p.post_mortem(tb)
 
         # usurp django's handler
-        from django.views import debug
-        debug.technical_500_response = postmortem if pm else null_technical_500_response
+        django_views_debug.technical_500_response = postmortem if pm else null_technical_500_response
 
         self.use_ipv6 = options['use_ipv6']
         if self.use_ipv6 and not socket.has_ipv6:
@@ -230,32 +293,28 @@ class Command(BaseCommand):
         """Return the default WSGI handler for the runner."""
         return get_internal_wsgi_application()
 
-    def get_systemcheckerror_handler(self, error_message, error_class=Exception, **options):
-        error_message = ansi_escape.sub('', error_message)
-
+    def get_error_handler(self, exc, **options):
         def application(env, start_response):
-            raise error_class(error_message)
+            if isinstance(exc, SystemCheckError):
+                error_message = ansi_escape.sub('', str(exc))
+                raise SystemCheckError(error_message)
+
+            raise exc
 
         return application
 
     def inner_run(self, options):
-        try:
-            from werkzeug import run_simple
-            from werkzeug.debug import DebuggedApplication
-            from werkzeug.serving import WSGIRequestHandler as _WSGIRequestHandler
-
-            # Set colored output
-            if settings.DEBUG:
-                try:
-                    set_werkzeug_log_color()
-                except Exception:  # We are dealing with some internals, anything could go wrong
-                    if self.show_startup_messages:
-                        print("Wrapping internal werkzeug logger for color highlighting has failed!")
-                    pass
-
-        except ImportError:
+        if not HAS_WERKZEUG:
             raise CommandError("Werkzeug is required to use runserver_plus.  Please visit http://werkzeug.pocoo.org/ or install via pip. (pip install Werkzeug)")
 
+        # Set colored output
+        if settings.DEBUG:
+            try:
+                set_werkzeug_log_color()
+            except Exception:  # We are dealing with some internals, anything could go wrong
+                if self.show_startup_messages:
+                    print("Wrapping internal werkzeug logger for color highlighting has failed!")
+
         class WSGIRequestHandler(_WSGIRequestHandler):
             def make_environ(self):
                 environ = super().make_environ()
@@ -266,7 +325,7 @@ class Command(BaseCommand):
         threaded = options['threaded']
         use_reloader = options['use_reloader']
         open_browser = options['open_browser']
-        quit_command = (sys.platform == 'win32') and 'CTRL-BREAK' or 'CONTROL-C'
+        quit_command = 'CONTROL-C' if sys.platform != 'win32' else 'CTRL-BREAK'
         reloader_interval = options['reloader_interval']
         reloader_type = options['reloader_type']
         self.extra_files = set(options['extra_files'])
@@ -277,14 +336,12 @@ class Command(BaseCommand):
             print("Performing system checks...\n")
 
         try:
-            self.check(display_num_errors=self.show_startup_messages)
-        except SystemCheckError as exc:
-            self.stderr.write("SystemCheckError occurred during system checks: " + str(exc), ending="\n\n")
-            handler = self.get_systemcheckerror_handler(str(exc), error_class=type(exc), **options)
-        else:
-            handler = self.get_handler(**options)
-
-        self.check_migrations()
+            check_errors(self.check)(display_num_errors=self.show_startup_messages)
+            check_errors(self.check_migrations)()
+            handler = check_errors(self.get_handler)(**options)
+        except Exception as exc:
+            self.stderr.write("Error occurred during checks: %r" % exc, ending="\n\n")
+            handler = self.get_error_handler(exc, **options)
 
         if USE_STATICFILES:
             use_static_handler = options['use_static_handler']
@@ -293,18 +350,7 @@ class Command(BaseCommand):
                 handler = StaticFilesHandler(handler)
 
         if options["cert_path"] or options["key_file_path"]:
-            """
-            OpenSSL is needed for SSL support.
-
-            This will make flakes8 throw warning since OpenSSL is not used
-            directly, alas, this is the only way to show meaningful error
-            messages. See:
-            http://lucumr.pocoo.org/2011/9/21/python-import-blackbox/
-            for more information on python imports.
-            """
-            try:
-                import OpenSSL  # NOQA
-            except ImportError:
+            if not HAS_OPENSSL:
                 raise CommandError("Python OpenSSL Library is "
                                    "required to use runserver_plus with ssl support. "
                                    "Install via pip (pip install pyOpenSSL).")
@@ -313,7 +359,6 @@ class Command(BaseCommand):
             dir_path, root = os.path.split(certfile)
             root, _ = os.path.splitext(root)
             try:
-                from werkzeug.serving import make_ssl_devcert
                 if os.path.exists(certfile) and os.path.exists(keyfile):
                     ssl_context = (certfile, keyfile)
                 else:  # Create cert, key files ourselves.
@@ -335,7 +380,6 @@ class Command(BaseCommand):
             print("Quit the server with %s." % quit_command)
 
         if open_browser:
-            import webbrowser
             webbrowser.open(bind_url)
 
         if use_reloader and settings.USE_I18N:
@@ -406,12 +450,8 @@ class Command(BaseCommand):
 
 def set_werkzeug_log_color():
     """Try to set color to the werkzeug log."""
-    from django.core.management.color import color_style
-    from werkzeug.serving import WSGIRequestHandler
-    from werkzeug._internal import _log
-
     _style = color_style()
-    _orig_log = WSGIRequestHandler.log
+    _orig_log = _WSGIRequestHandler.log
 
     def werk_log(self, type, message, *args):
         try:
@@ -444,4 +484,4 @@ def set_werkzeug_log_color():
 
         _log(type, msg)
 
-    WSGIRequestHandler.log = werk_log
+    _WSGIRequestHandler.log = werk_log
